package ru.dbotthepony.kstarbound.defs.liquid

import com.google.common.collect.ImmutableList
import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.io.json.builder.FactoryAdapter
import ru.dbotthepony.kstarbound.registerTypeAdapter
import ru.dbotthepony.kvector.vector.Color

data class LiquidDefinition(
	val name: String,
	val liquidId: Int,
	val description: String = "...",
	val tickDelta: Int = 1,
	val color: Color,
	val itemDrop: String? = null,
	val statusEffects: ImmutableList<String> = ImmutableList.of(),
	val interactions: ImmutableList<Interaction> = ImmutableList.of(),
	val texture: String,
	val bottomLightMix: Color,
	val textureMovementFactor: Double,
) {
	data class Interaction(val liquid: Int, val liquidResult: Int? = null, val materialResult: String? = null) {
		init {
			require(liquidResult != null || materialResult != null) { "Both liquidResult and materialResult are missing" }
		}
	}

	companion object {
		val ADAPTER = FactoryAdapter.Builder(LiquidDefinition::class)
			.auto(LiquidDefinition::name)
			.auto(LiquidDefinition::liquidId)
			.auto(LiquidDefinition::description)
			.auto(LiquidDefinition::tickDelta)
			.auto(LiquidDefinition::color)
			.auto(LiquidDefinition::itemDrop)
			.auto(LiquidDefinition::statusEffects)
			.auto(LiquidDefinition::interactions)
			.auto(LiquidDefinition::texture)
			.auto(LiquidDefinition::bottomLightMix)
			.auto(LiquidDefinition::textureMovementFactor)

		val INTERACTION_ADAPTER = FactoryAdapter.Builder(Interaction::class)
			.auto(Interaction::liquid)
			.auto(Interaction::liquidResult)
			.auto(Interaction::materialResult)

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapterFactory(ADAPTER)
			gsonBuilder.registerTypeAdapterFactory(INTERACTION_ADAPTER)
		}
	}
}
